-- Use // mod.db.profile // to interact with other options within the module
-- Use // HideBlizzard.db.profile.modules // to interact with main options

local mod = HideBlizzard:NewModule('Buttons', 'AceEvent-3.0')
mod.modName = 'Buttons'

local options = {
	chatbuttons = {
		type = 'toggle',
		name = 'Chat Buttons',
		width = 'full',
		get = function() return mod.db.profile.chatbuttons end,
		set = function(info, v)
			mod.db.profile.chatbuttons = v
			if v then
				mod:EnableChatButtons()
			else
				mod:DisableChatButtons()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Buttons end,
	},
	battleground = {
		type = 'toggle',
		name = 'Battleground',
		width = 'full',
		get = function() return mod.db.profile.battleground end,
		set = function(info, v)
			mod.db.profile.battleground = v
			if v then
				mod:EnableBattleground()
			else
				mod:DisableBattleground()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Buttons end,
	},
	calendar = {
		type = 'toggle',
		name = 'Calendar',
		width = 'full',
		get = function() return mod.db.profile.calendar end,
		set = function(info, v)
			mod.db.profile.calendar = v
			if v then
				mod:EnableCalendar()
			else
				mod:DisableCalendar()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Buttons end,
	},
	mail = {
		type = 'toggle',
		name = 'Mail',
		width = 'full',
		get = function() return mod.db.profile.mail end,
		set = function(info, v)
			mod.db.profile.mail = v
			if v then
				mod:EnableMail()
			else
				mod:DisableMail()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Buttons end,
	},
	tracking = {
		type = 'toggle',
		name = 'Tracking',
		width = 'full',
		get = function() return mod.db.profile.tracking end,
		set = function(info, v)
			mod.db.profile.tracking = v
			if v then
				mod:EnableTracking()
			else
				mod:DisableTracking()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Buttons end,
	},
	voice = {
		type = 'toggle',
		name = 'Voice',
		width = 'full',
		get = function() return mod.db.profile.voice end,
		set = function(info, v)
			mod.db.profile.voice = v
			if v then
				mod:EnableVoice()
			else
				mod:DisableVoice()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Buttons end,
	},
	world = {
		type = 'toggle',
		name = 'World Map',
		width = 'full',
		get = function() return mod.db.profile.world end,
		set = function(info, v)
			mod.db.profile.world = v
			if v then
				mod:EnableWorld()
			else
				mod:DisableWorld()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Buttons end,
	},
	zoom = {
		type = 'toggle',
		name = 'Zoom',
		width = 'full',
		get = function() return mod.db.profile.zoom end,
		set = function(info, v)
			mod.db.profile.zoom = v
			if v then
				mod:EnableZoom()
			else
				mod:DisableZoom()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Buttons end,
	},
}

local defaults = {
	profile = {
		chatbuttons = false,
		battleground = false,
		calender = false,
		mail = false,
		tracking = false,
		voice = false,
		world = false,
		zoom = false
	}
}

function mod:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace('Buttons', defaults)
end

function mod:OnEnable()
	self:UpdateFrames()
end

function mod:OnDisable()
	self:DisableChatButtons()
	self:DisableBattleground()
	self:DisableCalendar()
	self:DisableMail()
	self:DisableTracking()
	self:DisableVoice()
	self:DisableWorld()
	self:DisableZoom()
end

function mod:UpdateFrames()
	if mod.db.profile.chatbuttons == true then
		self:EnableChatButtons()
	else
		self:DisableChatButtons()
	end
	if mod.db.profile.battleground == true then
		self:EnableBattleground()
	else
		self:DisableBattleground()
	end
	if mod.db.profile.calendar == true then
		self:EnableCalendar()
	else
		self:DisableCalendar()
	end
	if mod.db.profile.mail == true then
		self:EnableMail()
	else
		self:DisableMail()
	end
	if mod.db.profile.tracking == true then
		self:EnableTracking()
	else
		self:DisableTracking()
	end
	if mod.db.profile.voice == true then
		self:EnableVoice()
	else
		self:DisableVoice()
	end
	if mod.db.profile.world == true then
		self:EnableWorld()
	else
		self:DisableWorld()
	end
	if mod.db.profile.zoom == true then
		self:EnableZoom()
	else
		self:DisableZoom()
	end
end

function mod:EnableChatButtons()
	ChatFrameMenuButton:Hide()
	for i = 1, NUM_CHAT_WINDOWS do
		local up = _G['ChatFrame'..i..'UpButton']
		up:Hide()
		up.Show = function() end
		local down = _G['ChatFrame'..i..'DownButton']
		down:Hide()
		down.Show = function() end
		local bottom = _G['ChatFrame'..i..'BottomButton']
		bottom:Hide()
		bottom.Show = function() end
	end
end

function mod:DisableChatButtons()
	ChatFrameMenuButton:Show()
	for i = 1, NUM_CHAT_WINDOWS do
		local up = _G['ChatFrame'..i..'UpButton']
		up.Show = nil
		up:Show()
		local down = _G['ChatFrame'..i..'DownButton']
		down.Show = nil
		down:Show()
		local bottom = _G['ChatFrame'..i..'BottomButton']
		bottom.Show = nil
		bottom:Show()
	end
end

function mod:EnableBattleground()
	BattlefieldFrame:UnregisterAllEvents()
	MiniMapBattlefieldFrame:Hide()
end

function mod:DisableBattleground()
	BattlefieldFrame_UpdateStatus()
end

function mod:EnableCalendar()
	GameTimeFrame:UnregisterAllEvents()
	GameTimeFrame:Hide()
end

function mod:DisableCalendar()
	GameTimeFrame:Show()
	GameTimeFrame:RegisterEvent('PLAYER_ENTERING_WORLD')
	GameTimeFrame:RegisterEvent('CALENDAR_UPDATE_PENDING_INVITES')
	GameTimeFrame:RegisterEvent('CALENDAR_EVENT_ALARM')
	GameTimeFrame:RegisterForClicks('AnyUp')
end

function mod:EnableMail()
	MiniMapMailFrame:UnregisterAllEvents()
	MiniMapMailFrame:Hide()
end

function mod:DisableMail()
	if HasNewMail() then
		MiniMapMailFrame:Show()
	else
		MiniMapMailFrame:Hide()
	end
end

function mod:EnableTracking()
	MiniMapTracking:UnregisterAllEvents()
	MiniMapTrackingButton:Hide()
	MiniMapTrackingIcon:Hide()
	MiniMapTrackingBackground:Hide()
end

function mod:DisableTracking()
	MiniMapTrackingButton:Show()
	MiniMapTrackingIcon:Show()
	MiniMapTrackingBackground:Show()
	MiniMapTrackingButton:RegisterEvent('MINIMAP_UPDATE_TRACKING')
end

function mod:EnableVoice()
	MiniMapVoiceChatFrame:UnregisterAllEvents()
	MiniMapVoiceChatFrame:Hide()
end

function mod:DisableVoice()
	MiniMapVoiceChat_Update()
end

function mod:EnableWorld()
	MiniMapWorldMapButton:Hide() 
end

function mod:DisableWorld()
	MiniMapWorldMapButton:Show()
end

function mod:EnableZoom()
	MinimapZoomIn:Hide()
	MinimapZoomOut:Hide()
end

function mod:DisableZoom()
	MinimapZoomIn:Show()
	MinimapZoomOut:Show()
end

function mod:Info()
	return 'Hides buttons connected to the chat and minimap frame.'
end

function mod:GetOptions()
	return options
end