-- Use // mod.db.profile // to interact with other options within the module
-- Use // HideBlizzard.db.profile.modules // to interact with main options

local mod = HideBlizzard:NewModule('SpecialFrames')
mod.modName = 'Special Frames'

local options = {
	minimap = {
		type = 'toggle',
		name = 'Minimap',
		width = 'full',
		get = function() return mod.db.profile.minimap end,
		set = function(info, v)
			mod.db.profile.minimap = v
			if v then
				mod:EnableMinimap()
			else
				mod:DisableMinimap()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.SpecialFrames end,
	},
	party = {
		type = 'toggle',
		name = 'Party',
		width = 'full',
		get = function() return mod.db.profile.party end,
		set = function(info, v)
			mod.db.profile.party = v
			if v then
				mod:EnableParty()
			else
				mod:DisableParty()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.SpecialFrames end,
	},
	tooltip = {
		type = 'toggle',
		name = 'Tooltip',
		width = 'full',
		get = function() return mod.db.profile.tooltip end,
		set = function(info, v)
			mod.db.profile.tooltip = v
			if v then
				mod:EnableTooltip()
			else
				mod:DisableTooltip()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.SpecialFrames end,
	},
	uierrorsframe = {
		type = 'toggle',
		name = 'UI Errors Frame',
		width = 'full',
		get = function() return mod.db.profile.uierrorsframe end,
		set = function(info, v)
			mod.db.profile.uierrorsframe = v
			if v then
				mod:EnableUIErrorsFrame()
			else
				mod:DisableUIErrorsFrame()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.SpecialFrames end,
	},
}

local defaults = {
	profile = {
		minimap = false,
		party = false,
		tooltip = false,
		uierrorsframe = false
	}
}

function mod:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace('Special Frames', defaults)
end

function mod:OnEnable()
	self:UpdateFrames()
end

function mod:OnDisable()
	self:DisableMinimap()
	self:DisableParty()
	self:DisableTooltip()
	self:DisableUIErrorsFrame()
end

function mod:UpdateFrames()
	if mod.db.profile.minimap == true then
		self:EnableMinimap()
	else
		self:DisableMinimap()
	end
	if mod.db.profile.party == true then
		self:EnableParty()
	else
		self:DisableParty()
	end
	if mod.db.profile.tooltip == true then
		self:EnableTooltip()
	else
		self:DisableTooltip()
	end
	if mod.db.profile.uierrorsframe == true then
		self:EnableUIErrorsFrame()
	else
		self:DisableUIErrorsFrame()
	end
end

function mod:EnableMinimap()
	MinimapCluster:Hide()
	MinimapCluster.Show = function() end
end

function mod:DisableMinimap()
	MinimapCluster.Show = nil
	MinimapCluster:Show()
end

function mod:EnableParty()
	for i = 1, MAX_PARTY_MEMBERS do
		local party = _G['PartyMemberFrame'..i]
		party:UnregisterAllEvents()
		party:Hide()
		party.Show = function() end
	end
	UIParent:UnregisterEvent('RAID_ROSTER_UPDATE')
end

function mod:DisableParty()
	for i = 1, MAX_PARTY_MEMBERS do
		local party = _G['PartyMemberFrame'..i]
		if (GetPartyMember(i)) then
			party.Show = nil
			party:Show()
			party:RegisterEvent('PARTY_MEMBERS_CHANGED')
			party:RegisterEvent('PARTY_LEADER_CHANGED')
			party:RegisterEvent('PARTY_MEMBER_ENABLE')
			party:RegisterEvent('PARTY_MEMBER_DISABLE')
			party:RegisterEvent('PARTY_LOOT_METHOD_CHANGED')
			party:RegisterEvent('MUTELIST_UPDATE')
			party:RegisterEvent('IGNORELIST_UPDATE')
			party:RegisterEvent('UNIT_PVP_UPDATE')
			party:RegisterEvent('UNIT_AURA')
			party:RegisterEvent('UNIT_PET')
			party:RegisterEvent('VARIABLES_LOADED')
			party:RegisterEvent('UNIT_NAME_UPDATE')
			party:RegisterEvent('UNIT_PORTRAIT_UPDATE')
			party:RegisterEvent('UNIT_DISPLAYPOWER')
			party:RegisterEvent('UNIT_ENTERED_VEHICLE')
			party:RegisterEvent('UNIT_EXITED_VEHICLE')
			party:RegisterEvent('VOICE_START')
			party:RegisterEvent('VOICE_STOP')
			party:RegisterEvent('VOICE_STATUS_UPDATE')
			party:RegisterEvent('READY_CHECK')
			party:RegisterEvent('READY_CHECK_CONFIRM')
			party:RegisterEvent('READY_CHECK_FINISHED')
			UnitFrame_OnEvent('PARTY_MEMBERS_CHANGED')
		end
	end
	UIParent:RegisterEvent('RAID_ROSTER_UPDATE')
end

local tooltipAddOns = {
	'CowTip',
	'lolTip',
	'nTooltip',
	'RantTooltip',
	'TinyTip',
	'Tipsy',
	'TipTac',
	'TipTop',
}

function mod:EnableTooltip()
	hooksecurefunc('GameTooltip_SetDefaultAnchor', function(tooltip, parent)
		for tooltipAddOns in pairs(tooltipAddOns) do
			if IsAddOnLoaded(tooltipAddOns) then
				return
			else
				tooltip:SetOwner(parent, 'ANCHOR_NONE')
				tooltip:Hide()
			end
		end
	end)
end

function mod:DisableTooltip()
	hooksecurefunc('GameTooltip_SetDefaultAnchor', function(tooltip, parent)
		for tooltipAddOns in pairs(tooltipAddOns) do
			if IsAddOnLoaded(tooltipAddOns) then
				return
			else
				tooltip:SetOwner(parent, 'ANCHOR_NONE')
				tooltip:SetPoint('BOTTOMRIGHT', 'UIParent', 'BOTTOMRIGHT', -CONTAINER_OFFSET_X - 13, CONTAINER_OFFSET_Y)
			end
		end
	end)
end

local errorAddOns = {
	'pError',
	'ErrorMonster',
}

function mod:EnableUIErrorsFrame()
	for errorAddOns in pairs(errorAddOns) do
		if IsAddOnLoaded(errorAddOns) then
			return
		else
			UIErrorsFrame:UnregisterAllEvents()
		end
	end
end

function mod:DisableUIErrorsFrame()
	for errorAddOns in pairs(errorAddOns) do
		if IsAddOnLoaded(errorAddOns) then
			return
		else
			UIErrorsFrame:RegisterEvent('SYSMSG')
			UIErrorsFrame:RegisterEvent('UI_INFO_MESSAGE')
			UIErrorsFrame:RegisterEvent('UI_ERROR_MESSAGE')
		end
	end
end

function mod:Info()
	return 'Hides various ui frames.'
end

function mod:GetOptions()
	return options
end