-- Use // mod.db.profile // to interact with other options within the module
-- Use // HideBlizzard.db.profile.modules // to interact with main options

local mod = HideBlizzard:NewModule('Vehicle', 'AceEvent-3.0')
mod.modName = 'Vehicle |cffEE2C2CBeta|r'

local options = {
	vehiclebuttons = {
		type = 'toggle',
		name = 'Vehicle Buttons',
		width = 'full',
		get = function() return mod.db.profile.vehiclebuttons end,
		set = function(info, v)
			mod.db.profile.vehiclebuttons = v
			if v then
				mod:EnableVehicleButtons()
			else
				mod:DisableVehicleButtons()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Vehicle end,
	},
	vehiclemenubar = {
		type = 'toggle',
		name = 'Vehicle Menu Bar',
		width = 'full',
		get = function() return mod.db.profile.vehiclemenubar end,
		set = function(info, v)
			mod.db.profile.vehiclemenubar = v
			if v then
				mod:EnableVehicleMenuBar()
			else
				mod:DisableVehicleMenuBar()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Vehicle end,
	},
	vehicleseat = {
		type = 'toggle',
		name = 'Vehicle Seat', 
		width = 'full',
		get = function() return mod.db.profile.vehicleseat end,
		set = function(info, v)
			mod.db.profile.vehicleseat = v
			if v then
				mod:EnableVehicleSeat()
			else
				mod:DisableVehicleSeat()
			end
		end,
		disabled = function() return not HideBlizzard.db.profile.modules.Vehicle end,
	},
}

local defaults = {
	profile = {
		vehiclebuttons = false,
		vehiclemenubar = false,
		vehicleseat = false
	}
}

function mod:OnInitialize()
	self.db = HideBlizzard.db:RegisterNamespace('Vehicle', defaults)
end

function mod:OnEnable()
	self:UpdateFrames()
end

function mod:OnDisable()
	self:DisableVehicleButtons()
	self:DisableVehicleMenuBar()
	self:DisableVehicleSeat()
end

function mod:UpdateFrames()
	if mod.db.profile.vehiclebuttons == true then
		self:EnableVehicleButtons()
	else
		self:DisableVehicleButtons()
	end
	if mod.db.profile.vehiclemenubar == true then
		self:EnableVehicleMenuBar()
	else
		self:DisableVehicleMenuBar()
	end
	if mod.db.profile.vehicleseat == true then
		self:EnableVehicleSeat()
	else
		self:DisableVehicleSeat()
	end
end

function mod:EnableVehicleButtons()
	VehicleMenuBar:UnregisterAllEvents()
	VehicleMenuBarPitchUpButton:Hide()
	VehicleMenuBarPitchDownButton:Hide()
	VehicleMenuBarLeaveButton:Hide()
end

function mod:DisableVehicleButtons()
	VehicleMenuBarPitchUpButton:Show()
	VehicleMenuBarPitchDownButton:Show()
	VehicleMenuBarLeaveButton:Show()
	VehicleMenuBar:RegisterEvent('UNIT_ENTERED_VEHICLE')
	VehicleMenuBar:RegisterEvent('VEHICLE_ANGLE_UPDATE')
end

function mod:EnableVehicleMenuBar()
	VehicleMenuBar:UnregisterAllEvents()
	VehicleMenuBar:Hide()
	VehicleMenuBar.Show = function() end
end

function mod:DisableVehicleMenuBar()
	VehicleMenuBar.Show = nil
	if UnitInVehicle('player') then
		VehicleMenuBar:Show()
		VehicleMenuBar:RegisterEvent('UNIT_ENTERING_VEHICLE')
		VehicleMenuBar:RegisterEvent('UNIT_ENTERED_VEHICLE')
		VehicleMenuBar:RegisterEvent('UNIT_EXITED_VEHICLE')
		VehicleMenuBar:RegisterEvent('UNIT_DISPLAYPOWER')
		VehicleMenuBar:RegisterEvent('VEHICLE_ANGLE_UPDATE')
		VehicleMenuBar:RegisterEvent('VEHICLE_PASSENGERS_CHANGED')
		VehicleMenuBar:RegisterEvent('PLAYER_GAINS_VEHICLE_DATA')
		VehicleMenuBar:RegisterEvent('PLAYER_LOSES_VEHICLE_DATA')
		VehicleMenuBar:RegisterEvent('PLAYER_ENTERING_WORLD')
	else
		VehicleMenuBar:Hide()
	end
end


function mod:EnableVehicleSeat()
	VehicleSeatIndicator:UnregisterAllEvents()
	VehicleSeatIndicator:Hide()
	VehicleSeatIndicator.Show = function() end
end

function mod:DisableVehicleSeat()
	VehicleSeatIndicator.Show = nil
	if UnitInVehicle('player') then
		VehicleSeatIndicator:Show()
		VehicleSeatIndicator:RegisterEvent('UNIT_ENTERING_VEHICLE')
		VehicleSeatIndicator:RegisterEvent('UNIT_ENTERED_VEHICLE')
		VehicleSeatIndicator:RegisterEvent('UNIT_EXITED_VEHICLE')
		VehicleSeatIndicator:RegisterEvent('UNIT_DISPLAYPOWER')
		VehicleSeatIndicator:RegisterEvent('VEHICLE_ANGLE_UPDATE')
		VehicleSeatIndicator:RegisterEvent('VEHICLE_PASSENGERS_CHANGED')
		VehicleSeatIndicator:RegisterEvent('PLAYER_GAINS_VEHICLE_DATA')
		VehicleSeatIndicator:RegisterEvent('PLAYER_LOSES_VEHICLE_DATA')
		VehicleSeatIndicator:RegisterEvent('PLAYER_ENTERING_WORLD')
	else
		VehicleSeatIndicator:Hide()
	end
end

function mod:Info()
	return 'Hides vehicle related frames.'
end

function mod:GetOptions()
	return options
end